# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.


"""Pytest configuration."""

from __future__ import absolute_import, print_function

import os
import shutil
import tempfile

import jinja2
import pytest
from flask import Flask
from flask_babelex import Babel
from helpers import make_fake_template
from invenio_assets import InvenioAssets
from invenio_i18n import InvenioI18N
from invenio_i18n.views import create_blueprint_from_app

from invenio_theme import InvenioTheme


@pytest.fixture()
def app():
    """Flask app fixture."""
    app = Flask("myapp")
    app.config.update(
        I18N_LANGUAGES=[("en", "English"), ("de", "German")],
    )
    Babel(app)
    InvenioI18N(app)
    app.register_blueprint(create_blueprint_from_app(app))
    return app


@pytest.fixture()
def app_error_handler(request):
    """Flask app error handler fixture."""
    app = Flask("myapp")

    # Creation of a fake theme error template file.
    temp_dir = make_fake_template(
        "{# -*- coding: utf-8 -*- -#}"
        "<!DOCTYPE html>{% block message %}"
        "{% endblock message %}"
    )
    # Adding the temporal path to jinja engine.
    app.jinja_loader = jinja2.ChoiceLoader(
        [jinja2.FileSystemLoader(temp_dir), app.jinja_loader]
    )

    # Setting by default fake.html as a THEME_ERROR_TEMPLATE
    app.config["THEME_ERROR_TEMPLATE"] = "invenio_theme/fake.html"

    # Tear down method to clean the temp directory.
    def tear_down():
        shutil.rmtree(temp_dir)

    request.addfinalizer(tear_down)

    app.testing = True
    Babel(app)
    InvenioI18N(app)
    InvenioTheme(app)
    return app


@pytest.fixture()
def app_frontpage_handler(request):
    """Flask app error handler fixture."""
    app = Flask("myapp")

    # Creation of a fake theme error template file.
    temp_dir = make_fake_template(
        "{% extends 'invenio_theme/page.html' %}"
        "{% block css %}{% endblock %}"
        "{% block javascript %}{% endblock %}"
    )

    # Adding the temporal path to jinja engine.
    app.jinja_loader = jinja2.ChoiceLoader(
        [jinja2.FileSystemLoader(temp_dir), app.jinja_loader]
    )

    # Setting by default fake.html as a BASE_TEMPLATE
    app.config["BASE_TEMPLATE"] = "invenio_theme/fake.html"
    app.config["THEME_FRONTPAGE"] = True

    # Tear down method to clean the temp directory.
    def tear_down():
        shutil.rmtree(temp_dir)

    request.addfinalizer(tear_down)

    app.testing = True
    Babel(app)
    InvenioI18N(app)
    InvenioTheme(app)
    InvenioAssets(app)
    return app



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio error handlers."""

from __future__ import absolute_import, print_function

from flask import Blueprint, current_app, render_template

blueprint = Blueprint("invenio_theme_frontpage", __name__)


@blueprint.route("/")
def index():
    """Simplistic front page view."""
    return render_template(
        current_app.config["THEME_FRONTPAGE_TEMPLATE"],
    )


def unauthorized(e):
    """Error handler to show a 401.html page in case of a 401 error."""
    return render_template(current_app.config["THEME_401_TEMPLATE"]), 401


def insufficient_permissions(e):
    """Error handler to show a 403.html page in case of a 403 error."""
    return render_template(current_app.config["THEME_403_TEMPLATE"]), 403


def page_not_found(e):
    """Error handler to show a 404.html page in case of a 404 error."""
    return render_template(current_app.config["THEME_404_TEMPLATE"]), 404


def too_many_requests(e):
    """Error handler to show a 429.html page in case of a 429 error."""
    return render_template(current_app.config["THEME_429_TEMPLATE"]), 429


def internal_error(e):
    """Error handler to show a 500.html page in case of a 500 error."""
    return render_template(current_app.config["THEME_500_TEMPLATE"]), 500



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2020 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""JS/CSS bundles for theme.

You include one of the bundles in a page like the example below (using
``base`` bundle as an example):

.. code-block:: html

    {{ webpack['base.js']}}

"""

from __future__ import absolute_import, print_function

from invenio_assets.webpack import WebpackThemeBundle

theme = WebpackThemeBundle(
    __name__,
    "assets",
    default="bootstrap3",
    themes={
        "bootstrap3": dict(
            entry={
                "base": "./js/invenio_theme/base.js",
                # requires jquery, moment, select2, bootstrap
                "adminlte": "./js/invenio_theme/admin.js",
                "theme-admin": "./scss/invenio_theme/admin.scss",
                "theme": "./scss/invenio_theme/theme.scss",
            },
            dependencies={
                "bootstrap-sass": "~3.3.5",
                "font-awesome": "~4.4.0",
                "jquery": "~3.2.1",
                "moment": "~2.23.0",
                "select2": "~4.0.2",
                "admin-lte": "~2.4.8",
            },
        ),
        "semantic-ui": dict(
            entry={
                "base": "./js/invenio_theme/base.js",
                # theme.js is the main file for the Semantic-UI theme.
                # - theme.js imports semantic-ui-less/semantic.less
                # - semantic-ui-less/semantic.less imports various files
                #   that all import "../../theme.config".
                # - theme.config (must be provided by an Invenio instance, that
                #   sets this as an alias) imports semantic-ui-less/theme.less.
                # - semantic-ui-less/theme.less imports
                #   1) the theme package: an invenio theme is configured below
                #      via the "themes/invenio" alias). theme.config set the
                #      respective variables to the "invenio" theme (e.g.
                #      "@site: 'invenio';").
                #   2) site theme: defined via the @siteFolder variable in
                #      "theme.config".
                "theme-admin": "./scss/invenio_theme/admin.scss",
                "adminlte": "./js/invenio_theme/admin.js",
                "theme": "./js/invenio_theme/theme.js",
            },
            dependencies={
                "semantic-ui-less": "~2.4.1",
                "semantic-ui-css": "~2.4.1",
                "font-awesome": "~4.4.0",
                "jquery": "~3.2.1",
                "select2": "~4.0.2",
                "admin-lte": "~2.4.8",
            },
            aliases={
                "@js/invenio_theme": "js/invenio_theme",
                "@less/invenio_theme": "less/invenio_theme",
                # Used for defining an 'invenio' theme for Semantic-UI. The
                # code in "Semantic-UI-Less/theme.less" will look for e.g.
                # themes/@{theme}/globals/site.variables". This will resolve
                # to the path "less/invenio_theme/theme/global/site.variables".
                # This means that you in "theme.config" can use "invenio" as a
                # theme, e.g.:
                #   @site: 'invenio';
                "themes/invenio": "less/invenio_theme/theme",
                # The @templates alias is used for overwriting JSX templates.
                # The alias assumes that a folder "templates" exists in the
                # root of the var/instance/assets (root of webpack project).
                # Cookiecutter-Invenio-Instance and Invenio-CLI is currently
                # responsible for creating this folder. The build will fail
                # if this folder is not created.
                "@templates": "templates",
            },
        ),
    },
)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Test for theme module."""

from __future__ import absolute_import, print_function

from flask import render_template_string
from invenio_assets import InvenioAssets

from invenio_theme import InvenioTheme


def assert_template_blocks(template, blocks, base_tpl=None):
    """Assert that blocks exists in template."""
    if base_tpl is None:
        base_tpl = """{%% extends '%s' %%}""" % template

    for b in blocks:
        tpl = base_tpl + "{%% block %s %%}TPLTEST{%% endblock %%}" % b
        assert "TPLTEST" in render_template_string(tpl)


def test_version():
    """Test version import."""
    from invenio_theme import __version__

    assert __version__


def test_init(app):
    """Initialization."""
    theme = InvenioTheme(app)
    assert theme.menu is not None
    assert "THEME_SITENAME" in app.config


def test_init_app(app):
    """Initialization."""
    theme = InvenioTheme()
    assert theme.menu is None
    theme.init_app(app)
    assert theme.menu is not None
    assert "THEME_SITENAME" in app.config


def test_render_template(app):
    """Test ability to render template."""
    # Remove assets to avoid problems compiling them.
    test_tpl = r"""{% extends 'invenio_theme/page.html' %}
    {% block css %}{% endblock %}
    {% block javascript %}{% endblock %}
    """

    InvenioTheme(app)
    InvenioAssets(app)
    with app.test_request_context():
        assert render_template_string(test_tpl)


def test_page_template_blocks(app):
    """Test template blocks in page.html."""
    base_tpl = r"""{% extends 'invenio_theme/page.html' %}
    {% block css %}{% endblock %}
    {% block javascript %}{% endblock %}
    """

    # Test template API
    blocks = [
        "head",
        "head_meta",
        "head_title",
        "head_links",
        "head_links_langs",
        "head_apple_icons",
        "header",
        "body",
        "browserupgrade",
        "page_header",
        "page_body",
        "page_footer",
        "trackingcode",
        "body_inner",
    ]
    InvenioTheme(app)
    InvenioAssets(app)

    with app.test_request_context():
        assert_template_blocks("invenio_theme/page.html", blocks, base_tpl=base_tpl)


def test_cover_template_blocks(app):
    """Test template blocks in page.html."""
    base_tpl = r"""{% extends 'invenio_theme/page_cover.html' %}
    {% set panel_title = 'Test' %}
    {% block css %}{% endblock %}
    {% block javascript %}{% endblock %}
    """

    # Test template API
    blocks = [
        "panel",
        "page_header",
        "page_body",
        "page_footer",
        "panel_content",
    ]
    InvenioTheme(app)
    InvenioAssets(app)

    with app.test_request_context():
        assert_template_blocks(
            "invenio_theme/page_cover.html", blocks, base_tpl=base_tpl
        )


def test_settings_template_blocks(app):
    """Test template blocks in page_settings.html."""
    base_tpl = r"""{% extends 'invenio_theme/page_settings.html' %}
    {% block css %}{% endblock %}
    {% block javascript %}{% endblock %}
    """

    blocks = ["page_body", "settings_menu", "settings_content", "settings_form"]
    InvenioTheme(app)
    InvenioAssets(app)

    with app.test_request_context():
        assert_template_blocks(
            "invenio_theme/page_settings.html", blocks, base_tpl=base_tpl
        )


def test_header_template_blocks(app):
    """Test template blokcs in header.html."""
    blocks = [
        "navbar",
        "navbar_header",
        "brand",
        "navbar_inner",
        "navbar_right",
        "breadcrumbs",
        "flashmessages",
        "navbar_nav",
        "navbar_search",
    ]
    InvenioTheme(app)
    InvenioAssets(app)
    with app.test_request_context():
        assert_template_blocks("invenio_theme/header.html", blocks)

    app.config.update(dict(THEME_SEARCHBAR=False))
    with app.test_request_context():
        tpl = (
            r'{% extends "invenio_theme/header.html" %}'
            r"{% block navbar_search %}TPLTEST{% endblock %}"
        )
        assert "TPLTEST" not in render_template_string(tpl)


def test_html_lang(app):
    """Test HTML language attribute."""
    base_tpl = r"""{% extends 'invenio_theme/page.html' %}
    {% block css %}{% endblock %}
    {% block javascript %}{% endblock %}
    """

    @app.route("/index")
    def index():
        """Render default page."""
        return render_template_string(base_tpl)

    InvenioTheme(app)
    InvenioAssets(app)

    with app.test_client() as client:
        response = client.get("/index")
        assert b'lang="en" ' in response.data

        response = client.get("/index?ln=de")
        assert b'lang="de" ' in response.data

        response = client.get("/index?ln=en")
        assert b'lang="en" ' in response.data


def test_frontpage_not_exists(app):
    """Test the frontpage that doesn't exist."""
    # Before configure the frontpage
    with app.test_client() as client:
        response = client.get("/")
        assert response.status_code == 404


def test_frontpage_exists(app_frontpage_handler):
    """Test the frontpage."""

    app_frontpage_handler.config.update(
        dict(
            THEME_FRONTPAGE_TITLE="Jessica Jones",
        )
    )

    # Check if exists
    with app_frontpage_handler.test_client() as client:
        response = client.get("/")
        assert b"Jessica Jones" in response.data



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

r"""Standard Bootstrap based theme for Invenio.

Invenio-Theme mainly consits of:

- `Templates and static files` - for integration with `Bootstrap
  <https://getbootstrap.com/>`_ HTML, CSS and JS framework.
- `Error handlers` - for showing user-friendly 401, 402, 404, and 500 error
  pages.
- `Menu and breadcrumbs` - for basic site navigation using `Flask-Menu
  <https://flask-menu.readthedocs.io/>`_ and `Flask-Breadcrumbs
  <https://flask-breadcrumbs.readthedocs.io/>`_.


Initialization
--------------
First create a Flask application and initialize
:class:`~invenio_theme.ext.InvenioTheme` extension.

>>> from flask import Flask, render_template_string
>>> from invenio_theme import InvenioTheme
>>> from flask_menu import register_menu
>>> app = Flask('myapp')
>>> theme = InvenioTheme(app)

.. note::

   There is a new blueprint registered during initialization in order to enable
   loading of templates and static files from Invenio-Theme. The blueprint does
   not  have any views registered.

In order for the following examples to work, you also need to load the
`Invenio-I18N <https://invenio-i18n.readthedocs.io>`_ and
`Invenio-Assets <https://invenio-assets.readthedocs.io>`_:

>>> from invenio_i18n import InvenioI18N
>>> from invenio_assets import InvenioAssets
>>> i18n = InvenioI18N(app)
>>> assets = InvenioAssets(app)

Basic customizations
--------------------
Invenio-Theme makes it easy to perform some simple customizations such as
changing logo.

Logo and name
~~~~~~~~~~~~~
The easiest customization you can do is changing the logo and name of your
Invenio instance. Simply set the :data:`invenio_theme.config.THEME_LOGO` and
:data:`invenio_theme.config.THEME_SITENAME` configuration variables:

>>> app.config.update({
... 'THEME_LOGO': 'images/...',
... 'THEME_SITENAME': 'Awesome Site',
... })

The logo path is relative to your static folders. Also, if you don't want a
logo displayed, you can simply set the value to ``None``.

Search bar
~~~~~~~~~~
If you don't like having the search bar available in the header, you can
modify the behaviour by setting
:data:`invenio_theme.config.THEME_SEARCHBAR` to ``False``:

>>> app.config.update({
... 'THEME_SEARCHBAR': False,
... })

Analytics tracking code
~~~~~~~~~~~~~~~~~~~~~~~
If you need to include an analyics tracking code in your HTML pages (e.g. to
track analytics with Piwik or Google Analytics) you should set the
:data:`invenio_theme.config.THEME_JAVASCRIPT_TEMPLATE` variable to a template
which includes the tracking code:

>>> app.config.update({
... 'THEME_JAVASCRIPT_TEMPLATE': 'invenio_theme/trackingcode.html',
... })

Note that ``invenio_theme/trackingcode.html`` is empty so you have to provide
your own template with the tracking code.

Google Site Verification
~~~~~~~~~~~~~~~~~~~~~~~~
`Google Webmasters Tools
<https://www.google.com/webmasters/tools/home?hl=en&pli=1>`_ allows you to
inspect how Google is indexing your site. In order to see your site, you must
provide proof of ownership which is done by e.g. including a token generated
by Google in the meta-tags of your website. You can easily do this with
Invenio-Theme theme. Simply update the
:data:`invenio_theme.config.THEME_GOOGLE_SITE_VERIFICATION` configuration
variable with the list of tokens provided by Google:

>>> app.config.update({
... 'THEME_GOOGLE_SITE_VERIFICATION': ['...token...'],
... })

Templates
---------
Templates in Invenio-Theme falls in two categories:

* **Page templates** such as the base, cover and settings templates which
  defines a template for an entire page.
* **Section templates** such as the header and footer templates which is
  included as part of a page templates.

Most simple visual changes can be done by just overriding a section template.
Larger visual changes will likely require changes to one or more of the
page templates.

Overriding templates
~~~~~~~~~~~~~~~~~~~~
Most of the time all you need is to customize a small part of an existing
template. This is best achieved by extending the Invenio-Theme template and
overriding the template block you would like to customize.

For instance, say you wanted to include two logo files instead of only one in
the header, then you could extend the header template and override the
``brand`` block like this:

.. code-block:: jinja

   {# mytheme/header.html #}
   {% extends "invenio_theme/header.html" %}
   {% block brand %} ... {% endblock brand %}


In order to make Invenio use this new header template you must set it in the
configuration (assuming the template was saved in a template folder under
``mytheme/header.html``):

>>> app.config.update({
... 'THEME_HEADER_TEMPLATE': 'mytheme/header.html',
... })

.. warning::

   In general, if you override a template you should provide the same template
   blocks as the template you override. This is because other Invenio modules
   may rely on these template blocks,

Header section template
~~~~~~~~~~~~~~~~~~~~~~~~
The header template (``invenio_theme/header.html``) is reponsible for rendering
the navigation bar (including logo, search bar, menu and login/sign up
buttons), flash messages and the breadcrumbs.

Change the template by updating
:data:`invenio_theme.config.THEME_HEADER_TEMPLATE`.

The template provide a number of blocks which can be overridden. The most
important ones are (please see the template file for full details):

* ``navbar`` - Displays the navigation bar and inlcude other template blocks
  such as ``navbar_header``, ``navbar_nav`` and ``navbar_right``.

* ``flashmessages`` - Displays small notification messages such as
  "Successfully logged in."

* ``breadcrumbs`` - Displays the breadcrumbs.

Header login section template
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
The header login template (``invenio_theme/header_login.html``) is responsible
for rendering the login/sign up buttons as well as a user menu if a user is
logged in. It is included by the header template and does not define any
template blocks.

Change the template by updating
:data:`invenio_theme.config.THEME_HEADER_LOGIN_TEMPLATE`

Footer section template
~~~~~~~~~~~~~~~~~~~~~~~
The footer template (``invenio_theme/footer.html``) is primarily reponsible for
rendering the language selector and does not contain any template blocks.

Change the template by updating
:data:`invenio_theme.config.THEME_FOOTER_TEMPLATE`

Javascript section template
~~~~~~~~~~~~~~~~~~~~~~~~~~~~
If all you need to customize is to load some additional JavaScript, you can
just override the JavaScript template (``invenio_theme/javascript.html``). This
template is responsible for including site-wide JavaScript modules.

Change the template by updating
:data:`invenio_theme.config.THEME_JAVASCRIPT_TEMPLATE`

Settings page template
~~~~~~~~~~~~~~~~~~~~~~
The settings template (``invenio_theme/page_settings.html``) is responsible for
rendering user settings pages such as the change password dialog.

.. image:: static/settings.png

Change the template by updating
:data:`invenio_theme.config.SETTINGS_TEMPLATE`

The template provide a number of blocks which can be overridden. The most
important ones are (please see the template file for full details):

* ``settings_menu`` - Renders the left-hand menu and include another template
  block ``settings_menu_item`` which renders an individual menu item.

* ``settings_content`` - Renders the right-hand content panel and include three
  template blocks for ``settings_content_title``, ``settings_body`` and
  ``settings_form``.

Cover page template
~~~~~~~~~~~~~~~~~~~
The cover template (``invenio_theme/page_cover.html``) has a simplified layout
and is used for e.g. displaying the login dialog and sign-up page.

.. image:: static/cover.png

Change the template by updating
:data:`invenio_theme.config.COVER_TEMPLATE`

The template provide a number of blocks which can be overridden. The most
important ones are (please see the template file for full details):

* ``page_header`` - Includes e.g. the logo and branding.

* ``page_body`` - Renders the page body which includes a panel. The panel can
  be customized via the template blocks ``panel`` and ``panel_content`` and by
  setting the template variable ``panel_title``.

* ``page_footer`` - Displayed right after the page body.

Base page template
~~~~~~~~~~~~~~~~~~
The base template (``invenio_theme/page.html``) is responsible for providing a
basic page skeleton with template blocks that can be overriden by templates
extending the base template.

.. warning::

   If you override this template it is very important that you provide the same
   template blocks as Invenio is relying on the template block API provided in
   this template.

.. image:: static/base.png

Change the template by updating
:data:`invenio_theme.config.BASE_TEMPLATE`

The template provide a number of blocks which can be overridden. The most
important ones are (please see the template file for full details):

* ``header`` - Empty block you can override to add extra content in the
  head-tag.

* ``css`` - Block containing the link to stylesheets. By default just the
  Invenio-Theme CSS is included. Placed in the head-tag.

* ``javascript`` - Block containg the JavaScript tags. By default just
  Invenio-Theme JS is included. Placed in the very end of the page.

* ``page_header`` - Block which includes the header section template.

* ``page_body`` - Block containing the page content. Normally when using the
  template, this is the only block you need to override.

* ``page_footer`` - Block which includes the footer section template.

When using the template, a couple of template variables can be set t

* ``title`` - Sets the header title-tag.

* ``description`` - Sets the description meta-tag.

* ``keywords`` - Sets the keywords meta-tag.

Error page templates
~~~~~~~~~~~~~~~~~~~~
The error template (``invenio_theme/page_error.html``) is used as basis for
error pages such as unauthorized access, internal server error etc.

.. image:: static/error.png

Change the template by updating
:data:`invenio_theme.config.THEME_ERROR_TEMPLATE`. If you wish to just override
individual error pages you can set the variables:

* :data:`invenio_theme.config.THEME_401_TEMPLATE`
* :data:`invenio_theme.config.THEME_403_TEMPLATE`
* :data:`invenio_theme.config.THEME_404_TEMPLATE`
* :data:`invenio_theme.config.THEME_500_TEMPLATE`

Front page
~~~~~~~~~~
The front page (``invenio_theme/frontpage.html``) is the homepage of the
application. By default is ``disabled``.

Enable the front page view by setting ``THEME_FRONTPAGE`` to ``True``
:data:`invenio_theme.config.THEME_FRONTPAGE`

Change the template by updating:
:data:`invenio_theme.config.THEME_FRONTPAGE_TEMPLATE`

The template provide a number of blocks which can be overridden. The most
important ones are (please see the template file for full details):

* ``page_header`` - Includes e.g. the logo and branding.

* ``page_body`` - Renders the page body which includes a panel. The panel can
  be customized via the template blocks ``panel`` and ``panel_content`` and by
  setting the template variable ``panel_title``.

Developing Invenio modules
--------------------------
.. note::

   This section is only relevant if you develop Invenio modules that need to be
   able to change style depending on the currently installed theme.

This section describes a set of patterns for Invenio modules that allows the
module to be themeable without hard depdencies on Invenio-Theme, which is
important as otherwise users of your module cannot change the default styling.

Base template pattern
~~~~~~~~~~~~~~~~~~~~~
The base template pattern allows your module to extend from the currently
installed theme's base template.

First, create a dummy base template in your module. The dummy base template
should define the same blocks as you plan to use from the Invenio-Theme
template. In most cases this means just providing the ``page_body`` block:

.. code-block:: jinja

    {# invenio_foo/base.html #}
    <html><body>
    {% block page_body %}{% endbxlock %}
    </body></html>

.. note::

    Template paths need to be globally unique, so normally you should put your
    templates in a subfolder named according to your module. For instance if
    you are developing Invenio-Foo, then put your base template in
    ``invenio_foo/base.html``.

Templates you develop in your module should extend from config variable instead
of directly from the base template:

.. code-block:: jinja

    {# invenio_foo/myview.html #}
    {% extends config.FOO_BASE_TEMPLATE %}
    {% block page_body %} ... {% endblock %}

The configuration variable you then set during your Flask extension
initialization, according to this pattern:

.. code-block:: python

    # invenio_foo/config.py
    FOO_BASE_TEMPLATE = 'invenio_foo/base.html'

.. code-block:: python

    # invenio_foo/ext.py
    from . import config

    class InvenioFoo(object):
        # ...
        def init_config(app):
            # Set FOO_BASE_TEMPLATE to the theme's
            # base template if theme is installed
            app.config.setdefault(
                'FOO_BASE_TEMPLATE',
                app.config.get('BASE_TEMPLATE'),
            )
            # If no theme is installed, this will set
            # the base template to the module's
            # base template.
            for k in dir(config):
                if k.startswith('FOO_'):
                    app.config.setdefault(
                        k,
                        getattr(config, k))

Template rendering pattern
~~~~~~~~~~~~~~~~~~~~~~~~~~
In views where you render a template you should normally let the template
path be defined by a configuration variable. Otherwise it is not possible for
an instance to override the template.

>>> from flask import current_app, render_template
>>> app.config.update({
...     'FOO_MYVIEW_TEMPLATE': 'invenio_foo/myview.html'
... })
>>> @app.route('/myview')
... def myview():
...     return render_template(
...         current_app.config['FOO_MYVIEW_TEMPLATE']
...     )

Alternatively if only some parts of a template should be customizable you can
apply the configration variable pattern to the include:

.. code-block:: jinja

    {% extends config.FOO_BASE_TEMPLATE %}
    {% block page_body %}
    {# ... #}
    {%- include config.FOO_MYVIEW_TEMPLATE %}
    {% endblock%}

Menus
~~~~~
Invenio-Theme defines are couple of menus that Invenio modules can plug into.
For instance if you want to plug an menu item in the navigation bar you can do
it like this:

>>> from flask_menu import register_menu
>>> @app.route('/myview2')
... @register_menu(app, 'main.myitem', 'My item', order=1)
... def myview2():
...     return ""

By default, any menu items that are children of ``main`` (such as the above
example) will be displayed on the header of every page, via the
``page_header`` block.

The following menus exists:

* ``main`` - Header menu navigation (defined by the base template).

* ``settings`` - User settings menu bar (defined by the settings template).

To read more about the creation and usage of menus, see :mod:`~flask_menu`.

Breadcrumbs
~~~~~~~~~~~
Breadcrumbs works similar to menus, just use the
:func:`~flask_breadcrumbs.register_breadcrumb` instead.

>>> from flask_breadcrumbs import register_breadcrumb

Using this decorator, you can specify the position of the view in a
hierarchical manner, as well as the title in the breadcrumb. By default, the
current breadcrumb is displayed inside the ``page_header`` block in the base
template.

>>> @app.route('/part1')
... @register_breadcrumb(app, '.', 'Index')
... def part1():
...     return ""
>>> @app.route('/part2')
... @register_breadcrumb(app, '.p2', 'Part 2')
... def part2():
...     return ""
>>> @app.route('/part3')
... @register_breadcrumb(app, '.p2.p3', 'Part 3')
... def part3():
...     return ""

To learn more about the usage of breadcrumbs, see :mod:`~flask_breadcrumbs`.

User settings
~~~~~~~~~~~~~
If your module allows your users to configure some settings, you can provide
a setting view that is plugged into the settings menu.

First, create your template for your view:

.. code-block:: jinja

    {% extends config.FOO_SETTINGS_TEMPLATE %}
    {% set panel_icon="fa fa-cog" %}
    {% set panel_title="Panel header" %}
    {% block settings_form %}
    ...
    {% endblock settings_form %}

Next, when creating the view register the view n the ``settings`` menu:

.. code-block:: python

    @app.route('/settings/')
    @register_breadcrumb(app, '.settings', 'Settings page')
    @register_menu(app, 'settings.item1', 'Item 1', order=2)
    def settings_item_1():
        return render_template('invenio_foo/foo_settings.html')

Message flashing
~~~~~~~~~~~~~~~~
At any time during a request, you may choose to record a message for the
user in order to display it only once in the future. This is a great way for
providing feedback to the user after an action, and Flask provides the method
:func:`flask.flash` for this purpose. The first parameter is the message, while
you may optionally supply a second parameter that serves as a category that
gives context to your message.

Invenio-Theme provides a macro in ``invenio_theme/macros/messages.html`` called
``flashed_messages`` that iterates through stored flash messages for the
current user and displays them in the current page. The category can be one of
'info', 'danger', 'warning' or 'success'. Using one of these will attach the
appropriate Bootstrap class to the message. By default, the current flashed
messages will be displayed inside the ``page_header`` block in the base
template.

.. code-block:: python

    @app.route('/myview')
    def settings_item_1():
        flash('Testing flashing', category='info')
        return render_template('...')
"""

from __future__ import absolute_import, print_function

from .ext import InvenioTheme

__version__ = "1.3.31"

__all__ = ("__version__", "InvenioTheme")
